@@ -11,7 +11,7 @@
 
 // receive events from the device
 // managed by the controller
-struct receiver {
+struct sc_receiver {
     sc_socket control_socket;
     sc_thread thread;
     sc_mutex mutex;
@@ -20,18 +20,18 @@ struct receiver {
 };
 
 bool
-receiver_init(struct receiver *receiver, sc_socket control_socket,
-              struct sc_acksync *acksync);
+sc_receiver_init(struct sc_receiver *receiver, sc_socket control_socket,
+                 struct sc_acksync *acksync);
 
 void
-receiver_destroy(struct receiver *receiver);
+sc_receiver_destroy(struct sc_receiver *receiver);
 
 bool
-receiver_start(struct receiver *receiver);
+sc_receiver_start(struct sc_receiver *receiver);
 
-// no receiver_stop(), it will automatically stop on control_socket shutdown
+// no sc_receiver_stop(), it will automatically stop on control_socket shutdown
 
 void
-receiver_join(struct receiver *receiver);
+sc_receiver_join(struct sc_receiver *receiver);
 
 #endif
